
//-------------------------------------------------------------------
// TITLE:   DIV guided missile - Tutorial
// VERSION: 1.0
// AUTHOR:  Avi Chami
// DATE:    31/01/2002
//-------------------------------------------------------------------


PROGRAM misile;

//-------------------------------------------------------------------
//               GENERAL NOTES TO THE PROGRAM
//
//  This tutorial shows how to program a guided missile,
//  with trailing smoke.
//
//  For this demo the enemy plane moves constantly in circles
//  and variable velocity
//
//  You can contact me at
//  chclau@yahoo.com should you have any questions.
//
// ******************************************************************
//
//  Please check my WIPs:
//
//     Lunar Lander
//     Space Duel
//     Armored Trains
//
//  And Game Over:
//
//     Metula
//
//  And / or visit my Games'n'Models site:
//
//     http://gnm5.tripod.com
//
//                           Avi (chclau)
//
//-------------------------------------------------------------------


//-------------------------------------------------------------------
GLOBAL

// Graphics and sound
    graph_id;                   // Id of the gfx
    explosion_id;               // Id of the explosion sound

// Processes variables
    enemy_process_flag;         // To check if there is an enemy plane
    enemy_id;                   // Id of the enemy process

// General purpose variable
    count;

BEGIN

//-------------------------------------------------------------------
// Initialize graphics & sound
//-------------------------------------------------------------------

    set_mode(m800x600);         // Sets video mode to 800x600
    set_fps(24,0);              // Number of frames per second


// Loads graphics
//
// This line asumes that the graphics sit under ...\div\fpg\mytut
// You can change the path to whichever directory you like
    graph_id = load_fpg("\mytut\missil.fpg");

// Load sound
//
// I used a sound from DIV's libraries
// Sound is not repetitive (0 parameter at the end)
    explosion_id = load_pcm("\library\war\explosio\explos11.pcm", 0);


// Fade out the screen while we draw the background
    fade (0,0,0,10);
    WHILE (fading) FRAME; END   // Waits while the screen fades off
    clear_screen();


    enemy_process_flag = FALSE;


// Call the player's plane process
    plane();





// Fade on
    fade (100,100,100,10);
    WHILE (fading) FRAME; END   // Waits while the screen fades on




//-------------------------------------------------------------------
// Main loop
//-------------------------------------------------------------------

    LOOP

        // On the main program we only check to see if there
        // is an enemy plane, and if not, we generate one

        IF (NOT enemy_process_flag)

            // Small delay until a new enemy plane is created
            FRAME (fps*500);

            enemy_process_flag = TRUE;
            enemy_id = enemy_plane();
        END

        FRAME;

    END

END

//-------------------------------------------------------------------
// Process plane
//   Takes care of the player's plane
//-------------------------------------------------------------------

PROCESS plane();

PRIVATE
    speed;
    fire_flag;

BEGIN

    x = 400;
    y = 300;
    z = -10;
    graph = 10;

    speed = 40;

    LOOP


        // Accelerate
        IF ( (key(_up)) AND (speed < 80) )
            speed++;
        END

        // Decelerate
        IF ( (key(_down)) AND (speed > 30) )
            speed--;
        END

        // Turn right
        IF ( key(_right) )
            angle -= 3000;
        END

        // Turn left
        IF ( key(_left) )
            angle += 3000;
        END

        // Fire! - with flag to avoid repeated shooting
        IF ( (key(_space)) AND (NOT fire_flag) )
            missile_pr (x,y,speed,angle);
            fire_flag = TRUE;
        END

        // Clear fire flag when user releases space key
        IF ( NOT (key(_space)) )
            fire_flag = FALSE;
        END



        // This is the movement sentence
        // Pay attention that we use 'speed' divided by 10
        advance(speed/10);


        // Check screen boundaries - and wrap around
        IF (x<0)
            x = 799;
        END

        IF (y<0)
            y = 599;
        END

        IF (x>799)
            x = 0;
        END

        IF (y>599)
            y = 0;
        END

        FRAME;
    END

END



//-------------------------------------------------------------------
// Process enemy plane
//   Takes care of the enemy plane
//-------------------------------------------------------------------

PROCESS enemy_plane();

PRIVATE
    speed;                          // plane speed
    selector;
    angle_delta;                    // change of speed
    speed_delta;                    // rate of change of plane angle

BEGIN

    x = 550;
    y = 300;

    angle = 90000;

    graph = 20;

    speed = 100;                    // plane speed
    speed_delta = 1;                // change of speed

    LOOP

        // Change angle permanently to move in circles
        angle = (angle + angle_delta) MOD 360000;
        speed += speed_delta;
        angle_delta = speed * 12;

        // Increase and decrease the speed cyclically
        IF (speed > 300)
            speed_delta = -1;
        END

        IF (speed < 80)
            speed_delta = 1;
        END

        // This is the movement sentence
        // Pay attention that we use 'speed' divided by 20
        advance(speed/20);

        // Check screen boundaries - and wrap around
        IF (x<0)
            x = 799;
        END

        IF (y<0)
            y = 599;
        END

        IF (x>799)
            x = 0;
        END

        IF (y>599)
            y = 0;
        END

        FRAME;
    END

END



//-------------------------------------------------------------------
// Process missile_pr
//   Takes care of the missiles
//
//
// Notes to this process:
//   The missile starts with the player's plane speed and
//   gradually accelerates until it reaches its maximum speed
//-------------------------------------------------------------------

PROCESS missile_pr(x,y,speed,angle)

PRIVATE
    speed;                          // plane speed
    angle2;                         // used for the missile guidance
    id_target;                      // used for collision detection

BEGIN

    graph = 100;
    z = 0;

    LOOP

        // This code guides the missile to the enemy plane
        // (only if there is an enemy plane on screen)
        IF (enemy_process_flag)

            // Calculate the angle of the enemy plane relative
            // to the missile
            angle2 = fget_angle(x, y, enemy_id.x, enemy_id.y);

            // Gradually change the missile angle to make it equal
            // to an angle that makes it collide with the enemy plane
            // '4000' is equal to four degrees
            // you can change this value to check its effects
            angle = near_angle (angle, angle2, 4000);

        END



        // Increase speed until the missile top speed is reached
        // Since speed is divided by '10' in the advance statement,
        // missile top speed is 15
        IF (speed < 150)
            speed++;
        END


        // Check collision withe enemy plane
        id_target = collision (TYPE enemy_plane);

        IF ( (id_target) AND (enemy_process_flag) )

            // explosion sound
            sound (explosion_id, 255, 255);

            enemy_process_flag = FALSE;

            // erase the missile
            graph = 0;

            // Draw the enemy plane explosion
            FROM count = 301 TO 307;
                enemy_id.graph = count;
                FRAME;
                FRAME;
            END

            // Kill the enemy plane process
            signal (id_target, s_kill);

            FRAME;

            break;
        END

        // This is the movement sentence
        // Pay attention that we use 'speed' divided by 10
        advance(speed/10);

        // Check screen boundaries - and kill missile if it crosses
        // them
        IF ( (x<0) OR (y<0) OR (x>799) OR (y>599) )
            break;
        END

        // Generate smoke
        // To generate smoke, we generate 'smoke' processes after
        // the missile (this is achieved by the get_distx/y functions)
        // This processes 'live' for a short time and then dissapear
        smoke(x+get_distx(angle,-20), y+get_disty(angle,-20));

        FRAME;
    END

END


//-------------------------------------------------------------------
// Process smoke
//   Draws the missile trailing smoke
//
// The smoke processes gradually get bigger,
// and after reaching a certain size (180), they start to
// shrink. The shrinking speed(10) is much bigger than that of
// the expansion (2).
//-------------------------------------------------------------------

PROCESS smoke(x,y)

PRIVATE
    delta;                      // Used for expanding/shrinking the process

BEGIN

    graph = 200;
    z     = 10;
    size  = 140;
    angle = rand (0,359000);
    delta = 2;

    LOOP

        size += delta;

        IF (size > 180)
            delta = -10;
        END

        IF ( (size < 70) AND (delta == -10));
            break;
        END

        FRAME;
    END

END
